@@ -1,5 +1,5 @@
-{-# LANGUAGE DeriveDataTypeable, OverloadedStrings, RecordWildCards,
-    ScopedTypeVariables #-}
+{-# LANGUAGE DeriveDataTypeable, DeriveGeneric, OverloadedStrings,
+    RecordWildCards, ScopedTypeVariables #-}
 
 -- |
 -- Module      : Criterion.Report
@@ -34,6 +34,7 @@ import Criterion.Config (cfgReport, cfgTemplate, fromLJ)
 import Criterion.Monad (Criterion, getConfig)
 import Data.Data (Data, Typeable)
 import Data.Monoid (Last(..))
+import GHC.Generics (Generic)
 import Paths_criterion (getDataFileName)
 import Statistics.Sample.KernelDensity (kde)
 import Statistics.Types (Sample)
@@ -41,10 +42,12 @@ import System.Directory (doesFileExist)
 import System.FilePath ((</>), isPathSeparator)
 import System.IO.Unsafe (unsafePerformIO)
 import Text.Hastache (MuType(..))
-import Text.Hastache.Context (mkGenericContext, mkStrContext)
+import Text.Hastache.Context (mkGenericContext, mkStrContext, mkStrContextM)
 import qualified Control.Exception as E
-import qualified Data.ByteString.Char8 as B
-import qualified Data.ByteString.Lazy as L
+import qualified Data.Text as T
+import qualified Data.Text.IO as T
+import qualified Data.Text.Lazy as TL
+import qualified Data.Text.Lazy.IO as TL
 import qualified Data.Vector.Generic as G
 import qualified Data.Vector.Unboxed as U
 import qualified Text.Hastache as H
@@ -55,7 +58,7 @@ data Report = Report {
     , reportTimes :: Sample
     , reportAnalysis :: SampleAnalysis
     , reportOutliers :: Outliers
-    } deriving (Eq, Show, Typeable, Data)
+    } deriving (Eq, Read, Show, Typeable, Data, Generic)
 
 -- | The path to the template and other files used for generating
 -- reports.
@@ -72,25 +75,25 @@ report reports = do
     Last Nothing -> return ()
     Last (Just name) -> liftIO $ do
       tpl <- loadTemplate [".",templateDir] (fromLJ cfgTemplate cfg)
-      L.writeFile name =<< formatReport reports tpl
+      TL.writeFile name =<< formatReport reports tpl
 
 -- | Format a series of 'Report' values using the given Hastache
 -- template.
 formatReport :: [Report]
-             -> B.ByteString    -- ^ Hastache template.
-             -> IO L.ByteString
+             -> T.Text    -- ^ Hastache template.
+             -> IO TL.Text
 formatReport reports template = do
-  let context "report"  = MuList $ map inner reports
-      context "include" = MuLambdaM $ includeFile [templateDir]
-      context _         = MuNothing
-      inner Report{..} = mkStrContext $ \nym ->
+  let context "report"  = return $ MuList $ map inner reports
+      context "include" = return $ MuLambdaM $ includeFile [templateDir]
+      context _         = return $ MuNothing
+      inner Report{..} = mkStrContextM $ \nym ->
                          case nym of
-                           "name"     -> MuVariable reportName
-                           "number"   -> MuVariable reportNumber
-                           "times"    -> vector "x" reportTimes
-                           "kdetimes" -> vector "x" kdeTimes
-                           "kdepdf"   -> vector "x" kdePDF
-                           "kde"      -> vector2 "time" "pdf" kdeTimes kdePDF
+                           "name"     -> return $ MuVariable reportName
+                           "number"   -> return $ MuVariable reportNumber
+                           "times"    -> return $ vector "x" reportTimes
+                           "kdetimes" -> return $ vector "x" kdeTimes
+                           "kdepdf"   -> return $ vector "x" kdePDF
+                           "kde"      -> return $ vector2 "time" "pdf" kdeTimes kdePDF
                            ('a':'n':_)-> mkGenericContext reportAnalysis $
                                          H.encodeStr nym
                            _          -> mkGenericContext reportOutliers $
@@ -152,18 +155,18 @@ vector2 name1 name2 v1 v2 = MuList $ zipWith val (G.toList v1) (G.toList v2)
 -- \"@/etc/passwd@\".
 includeFile :: (MonadIO m) =>
                [FilePath]       -- ^ Directories to search.
-            -> B.ByteString     -- ^ Name of the file to search for.
-            -> m B.ByteString
-{-# SPECIALIZE includeFile :: [FilePath] -> B.ByteString -> IO B.ByteString #-}
-includeFile searchPath name = liftIO $ foldr go (return B.empty) searchPath
+            -> T.Text          -- ^ Name of the file to search for.
+            -> m T.Text
+{-# SPECIALIZE includeFile :: [FilePath] -> T.Text -> IO T.Text #-}
+includeFile searchPath name = liftIO $ foldr go (return T.empty) searchPath
     where go dir next = do
             let path = dir </> H.decodeStr name
-            B.readFile path `E.catch` \(_::IOException) -> next
+            T.readFile path `E.catch` \(_::IOException) -> next
 
 -- | A problem arose with a template.
 data TemplateException =
     TemplateNotFound FilePath   -- ^ The template could not be found.
-    deriving (Eq, Show, Typeable, Data)
+    deriving (Eq, Read, Show, Typeable, Data, Generic)
 
 instance Exception TemplateException
 
@@ -176,15 +179,15 @@ instance Exception TemplateException
 -- not be found, or an 'IOException' if no template could be loaded.
 loadTemplate :: [FilePath]      -- ^ Search path.
              -> FilePath        -- ^ Name of template file.
-             -> IO B.ByteString
+             -> IO T.Text
 loadTemplate paths name
-    | any isPathSeparator name = B.readFile name
+    | any isPathSeparator name = T.readFile name
     | otherwise                = go Nothing paths
   where go me (p:ps) = do
           let cur = p </> name
           x <- doesFileExist cur
           if x
-            then B.readFile cur `E.catch` \e -> go (me `mplus` Just e) ps
+            then T.readFile cur `E.catch` \e -> go (me `mplus` Just e) ps
             else go me ps
         go (Just e) _ = throwIO (e::IOException)
         go _        _ = throwIO . TemplateNotFound $ name
